#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""==============================================================================
# Project: MSIRNA
# Script : train.py
# Author : Peng Jia
# Date   : 2021.03.28
# Email  : pengjia@stu.xjtu.edu.cn
# Description: TODO
=============================================================================="""
import os
import pandas as pd
import numpy as np
import pickle
from sklearn import svm, datasets
from sklearn.metrics import *
from sklearn.neural_network import MLPClassifier
from sklearn.naive_bayes import GaussianNB
from sklearn.ensemble import RandomForestClassifier
from sklearn.ensemble import AdaBoostClassifier
from sklearn.linear_model import LogisticRegression
from imblearn.over_sampling import RandomOverSampler, SMOTE
from collections import Counter
from src.units import logger


def build_classrfier(type="RandomForest"):
    random_state = np.random.RandomState(1)
    if type == "SVM":
        classifier = svm.SVC(kernel='linear', probability=True,
                             random_state=random_state)
    elif type == "LogisticRegression":
        classifier = LogisticRegression(C=1.0, penalty='l1', tol=0.0001, solver='saga')
    elif type == "MLPClassifier":
        classifier = MLPClassifier(solver='lbfgs', alpha=1e-6,
                                   hidden_layer_sizes=(20, 10, 5), random_state=1)
    elif type == "GaussianNB":
        classifier = GaussianNB()
    elif type == "RandomForest":
        classifier = RandomForestClassifier(n_estimators=700, oob_score=True)
    elif type == "AdaBoostClassifier":
        classifier = AdaBoostClassifier(n_estimators=1000)

    return classifier


def train_model(paras):
    input = paras.input[0]
    model_path = paras.model[0]
    cancer_type = paras.cancer_type[0]
    classifier_name = paras.classifier[0]
    input_description = paras.input_description[0]
    model_description = paras.model_description[0]
    positive_num = paras.positive_num[0]
    author = paras.author[0]
    email = paras.email[0]

    input_df = pd.read_csv(input, index_col=0).dropna()
    genes = input_df.columns.to_list()
    genes.remove("msi")
    msi_status = input_df["msi"].to_list()
    # print(msi_status)
    y_label = [1 if i.upper() in ["MSI", "MSI-H", "MSI_H"] else 0 for i in msi_status]
    class_num = Counter(y_label)
    if class_num[1] < positive_num:
        logger.error("The No. of MSI sample lower than the minimum values, Please set with -p.")
    x = input_df[genes]
    smo = SMOTE(random_state=1)
    x_balanced, y_label_balanced = smo.fit_resample(x, y_label)
    classifier = build_classrfier(type=classifier_name)
    classifier.fit(x_balanced, y_label_balanced)

    y_pre = classifier.predict(x)
    y_pre_pro = classifier.predict_proba(x)[:, 1]
    roc_auc = roc_auc_score(y_label, y_pre_pro)
    conf_matrix = confusion_matrix(y_label, y_pre, labels=None, sample_weight=None)
    TN = conf_matrix[0][0]
    FN = conf_matrix[1][0]
    TP = conf_matrix[1][1]
    FP = conf_matrix[0][1]
    sensitivity = TP / (TP + FN)
    precision = TP / (TP + FP)
    specificity = TN / (TN + FP)
    f_score1 = f1_score(y_label, y_pre)
    accuracy_score = (TP + TN) / (FN + FP + TN + TP)

    description = {}
    description["Cancer Type"] = cancer_type
    description["Classifier"] = classifier_name
    description["Input Description"] = input_description
    description["Model Description"] = model_description
    description["Available Trained Sample"] = len(input_df)
    description["Positive Sample"] = class_num[1]
    description["Negative Sample"] = class_num[0]
    description["Accuracy"] = accuracy_score
    description["AUC"] = roc_auc
    description["F1 Score"] = f_score1
    description["Sensitivity"] = sensitivity
    description["Specificity"] = specificity
    description["Precision"] = precision
    description["Model Path"] = os.path.abspath(model_path)
    description["Author"] = os.path.abspath(author)
    description["Email"] = os.path.abspath(email)
    for item, value in description.items():
        logger.info(item + ": {}".format(value))
    with open(model_path, 'wb') as f:
        pickle.dump(classifier, f)
        pickle.dump(description, f)
        pickle.dump(genes, f)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""==============================================================================
# Project: MSIRNA
# Script : msisensor.py
# Author : Peng Jia
# Date   : 2021.03.19
# Email  : pengjia@stu.xjtu.edu.cn
# Description: TODO
=============================================================================="""
import sys
import os

# sys.path.append("/public/ProjectBaseCall/code/ProjectBaseCall/0_data_preprocessing")
curpath = os.path.abspath(os.path.dirname(sys.argv[0]))
sys.path.append(os.path.dirname(curpath))

from src.paras import *
from src.detection import detection_msi
from src.train import train_model
from src.show import show_model


def main():
    paras = args_process()
    if paras:
        # print(paras)
        if paras.command == "detection":
            detection_msi(paras)
        elif paras.command == "train":
            train_model(paras)
        elif paras.command == "show":
            show_model(paras)

    pass


if __name__ == "__main__":
    # print("hhh")
    main()
    pass



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""==============================================================================
# Project: MSIRNA
# Script : paras.py
# Author : Peng Jia
# Date   : 2021.03.19
# Email  : pengjia@stu.xjtu.edu.cn
# Description: parameters of the software
=============================================================================="""
import logging

version_num = "1.0"  # build at 2021.03.19
paras_default = {

}

import os
import sys

curpath = os.path.abspath(os.path.dirname(sys.argv[0]))
sys.path.append(os.path.dirname(curpath))
import argparse
from src.units import *


def args_process():
    """
    argument procress
    """
    commands = []
    commands_parser = {}
    parser = argparse.ArgumentParser(description=default_general["name"] + ": " +
                                                 default_general["description"]
                                     )
    parser.usage = default_general["name"] + " <command> [options]"
    parser.add_argument('-V', '--version', action='version',
                        version=default_general["name"] + default_general["version"])
    subparsers = parser.add_subparsers(title="command", metavar="", dest='command')

    ###################################################################################################################
    # command detection
    parser_detection = subparsers.add_parser('detection', help='Microsatellite instability detection.')
    parser_detection.description = 'Microsatellite instability detection.'
    commands.append("detection")
    parser_detection.add_argument('-i', '--input', required=True, type=str, nargs=1,
                                  help="The path of input file [required]")
    parser_detection.add_argument('-o', '--output', required=True, type=str, nargs=1,
                                  help="The path of output file prefix [required]")
    parser_detection.add_argument('-m', '--model', required=True, type=str, nargs=1,
                                  help="The path of the microsatellite regions [required]")
    parser_detection.add_argument('-d', '--run_directly', required=False, type=bool, default=[False], nargs=1,
                                  help="Run the program directly without any Confirm. [default=False]")

    commands_parser["detection"] = parser_detection
    ###################################################################################################################
    # command train
    parser_train = subparsers.add_parser('train',
                                         help='Train custom model for microsatellite instability detection.')
    parser_train.description = 'Train custom model for microsatellite instability detection.'
    commands.append("train")
    parser_train.add_argument('-i', '--input', required=True, type=str, nargs=1,
                              help="The path of input file. [required]")
    parser_train.add_argument('-m', '--model', required=True, type=str, nargs=1,
                              help="The trained model of the input file. [required]")
    parser_train.add_argument("-t", "--cancer_type", required=True, type=str, nargs=1,
                              help="The cancer type for this training. e.g. CRC, STAD, PanCancer etc. ")
    parser_train.add_argument('-c', '--classifier', required=False, type=str, nargs=1, default=["RandomForest"],
                              choices=["RandomForest", "LogisticRegression", "MLPClassifier", "GaussianNB",
                                       "AdaBoostClassifier"],
                              help="The machine learning classifier for MSI detection. [default=RandomForest]")
    parser_train.add_argument('-di', '--input_description', required=False, type=str, nargs=1, default=["."],
                              help="The description of the input file.")
    parser_train.add_argument("-dm", "--model_description", required=False, type=str, nargs=1,
                              default=["."],
                              help="Description for this trained model.")
    parser_train.add_argument("-p", "--positive_num", required=False, type=int, nargs=1,
                              default=[10],
                              help="The minimum  positive sample of MSI for training. [default=10]")
    parser_train.add_argument("-a", "--author", required=False, type=str, nargs=1,
                              default=["."],
                              help="The author who trained the model.")
    parser_train.add_argument("-e", "--email", required=False, type=str, nargs=1,
                              default=["."],
                              help="The email of the author.")
    commands_parser["train"] = parser_train

    ###################################################################################################################
    # command train
    parser_show = subparsers.add_parser('show', help='Show the information of the model and add more details.')
    parser_show.description = 'Show the information of the model and add more details.'
    commands.append("show")

    parser_show.add_argument('-m', '--model', required=True, type=str, nargs=1,
                             help="The trained model path. [required]")
    parser_show.add_argument("-t", "--cancer_type", required=False, type=str, nargs=1, default=["."],
                             help="Rename the cancer type. e.g. CRC, STAD, PanCancer etc. ")
    parser_show.add_argument('-di', '--input_description', required=False, type=str, nargs=1, default=["."],
                             help="Add description for the input file.")
    parser_show.add_argument("-dm", "--model_description", required=False, type=str, nargs=1,
                             default=["."],
                             help="Add description for this trained model.")
    parser_show.add_argument("-g", "--gene_list", required=False, type=str, nargs=1,
                             default=["."],
                             help="The path for the genes must be included for this model.")

    commands_parser["show"] = parser_show

    if len(os.sys.argv) < 2:
        parser.print_help()
        return False

    if os.sys.argv[1] in ["-h", "--help", "-?", "--h", "-H", "--H"]:
        parser.print_help()
        return False
    if os.sys.argv[1] in ["-V", "-v", "--version", "-version", "--v"]:
        # parser.print_help()
        # parser.parse_args("-V")
        print(default_general["name"] + " " + default_general["version"])
        return False
    if os.sys.argv[1] not in commands:
        logger.error("Command Error! " + os.sys.argv[1] +
                     " is not the available command.\n"
                     "[Tips] Please input correct command such as " + ", ".join(commands) + "!")
        return False
    if len(os.sys.argv) == 2 and (os.sys.argv[1] in commands):
        commands_parser[os.sys.argv[1]].print_help()
        return False
    return parser.parse_args()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""==============================================================================
# Project: MSIRNA
# Script : units.py
# Author : Peng Jia
# Date   : 2021.03.19
# Email  : pengjia@stu.xjtu.edu.cn
# Description: TODO
=============================================================================="""
import logging

logger = logging.getLogger()
logger.setLevel(logging.DEBUG)
consoleHandler = logging.StreamHandler()
consoleHandler.setLevel(logging.DEBUG)
formatter = logging.Formatter('[%(levelname)s]\t %(message)s')
consoleHandler.setFormatter(formatter)
logger.addHandler(consoleHandler)

default_general = {
    "version": "0.1.2",
    "name": "msisensor-rna",
    "description": "Microsatellite Instability (MSI) detection with RNA sequencing data.",
    "author": "Peng Jia, Xuanhao Yang et al.",
    "contact": "pengjia@stu.xjtu.edu.cn"
}
default_detection = {
    "custom_model": False
}



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""==============================================================================
# Project: MSIRNA
# Script : show.py
# Author : Peng Jia
# Date   : 2021.03.29
# Email  : pengjia@stu.xjtu.edu.cn
# Description: TODO
=============================================================================="""

import pickle

from src.units import logger


def show_model(paras):
    model_path = paras.model[0]
    cancer_type = paras.cancer_type[0]
    input_description = paras.input_description[0]
    model_description = paras.model_description[0]
    gene_list_path = paras.gene_list[0]
    with open(model_path, 'rb') as f:
        classifier = pickle.load(f)
        description = pickle.load(f)
        genes = pickle.load(f)
    write_stat = False
    if cancer_type != ".":
        description["Cancer Type"] = cancer_type
        write_stat = True
    elif input_description != ".":
        description["Input Description"] = input_description
        write_stat = True
    elif model_description != ".":
        description["Model Description"] = model_description
        write_stat = True
    if write_stat:
        with open(model_path, 'wb') as f:
            pickle.dump(classifier, f)
            pickle.dump(description, f)
            pickle.dump(genes, f)
    for item, value in description.items():
        logger.info(item + ": {}".format(value))
    if gene_list_path != ".":
        with open(gene_list_path, "w") as f:
            f.write("\n".join(genes))



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""==============================================================================
# Project: MSIRNA
# Script : detection.py
# Author : Peng Jia
# Date   : 2021.03.22
# Email  : pengjia@stu.xjtu.edu.cn
# Description: TODO
=============================================================================="""
import pickle
import pandas as pd
from src.units import logger


def detection_msi(paras):
    input_path = paras.input[0]
    output_path = paras.output[0]
    model_path = paras.model[0]
    directly_run = paras.run_directly[0]
    logger.info("The input expression data: {}".format(input_path))
    logger.info("The output MSI result: {}".format(output_path))
    logger.info("The model path: {}".format(model_path))
    with open(model_path, 'rb') as f:
        classifier = pickle.load(f)
        description = pickle.load(f)
        genes = pickle.load(f)
    logger.info("======================================================")
    for item, value in description.items():
        logger.info("Model description -- " + item + ": {}".format(value))
    if not directly_run:
        run_next = input("Please check the Model description! Are you sure to use this model to predict MSI? [Yes/No] ")
        if run_next.upper() not in ["YES", "Y"]:
            logger.info("Exit by the user")
    logger.info("Check the input file!")
    df_input = pd.read_csv(input_path, index_col=0).fillna(0)
    genes_this = df_input.columns.to_list()
    if len(set(genes) - set(genes_this)) > 0:
        logger.error("Please provide the fallowing genes' expression!")
        print(",".join(set(genes) - set(genes_this)))
        exit()
    x = df_input[genes]
    x_name = df_input.index.to_list()
    y_pre = classifier.predict(x)
    y_pre_pro = classifier.predict_proba(x)[:, 1]
    output_file = open(output_path, "w")
    output_file.write("sample_id,msi_status,probability_of_MSI-H")
    for sample, msi, msi_pro in zip(x_name, y_pre, y_pre_pro):
        output_file.write("{sample},{msi},{msi_pro}\n".format(sample=sample, msi="MSI-H" if msi == 1 else "MSI",
                                                              msi_pro=round(msi_pro, 4)))
    output_file.close()
    logger.info("Finished!")



# !/usr/bin/env python
# -*- coding: UTF-8 -*-
"""==============================================================================
# Project: MSIRNA
# Script : test.py
# Author : Peng Jia
# Date   : 2021.03.28
# Email  : pengjia@stu.xjtu.edu.cn
# Description: TODO
=============================================================================="""
import pandas as pd

df = pd.read_csv("/mnt/project/ProjectMSI/MSI_RNA/software/MSIRNA/demo/TCGA.STAD.independent.mean.100.csv", index_col=0)
df_case = pd.read_csv("/mnt/project/ProjectMSI/MSI_RNA/software/MSIRNA/demo/caseinfo.TCGA.STAD.csv", index_col=0)
df["msi"] = df_case["msi"]
df.to_csv("/mnt/project/ProjectMSI/MSI_RNA/software/MSIRNA/demo/input.csv")

print(df)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""==============================================================================
# Project: msisensor-rna
# Script : setup.py.py
# Author : Peng Jia
# Date   : 2021.03.29
# Email  : pengjia@stu.xjtu.edu.cn
# Description: TODO
=============================================================================="""
from setuptools import setup

from src.units import default_general

setup(
    name=default_general["name"],
    version=default_general["version"],
    description=default_general["name"] + " " + default_general["description"],
    url="https://github.com/xjtu-omics/msisensor-rna",
    author=default_general["author"],
    author_email=default_general["contact"],
    license='Custom License',
    keywords='Microsatellite instability, RNA-seq, Immunotherapy, Tumor only ',
    packages=['src'],
    install_requires=['pandas>=1.0', "numpy==1.16",
                      "scikit-learn>=0.24", "imbalanced-learn>=0.8.0"],
    python_requires='>=3.6',
    entry_points={'console_scripts': ['msisensor-rna = src.msisensor:main']},
)
